from tap_amazon_mws.streams.base import PaginatedStream, pluck, get_price
from tap_amazon_mws.state import incorporate, save_state
from dateutil.parser import parse

import singer
import time


LOGGER = singer.get_logger()  # noqa


class OrdersStream(PaginatedStream):
    TABLE = 'orders'
    KEY_PROPERTIES = ['id']

    def __init__(self, *args, **kwargs):
        super(OrdersStream, self).__init__(*args, **kwargs)

    def get_config(self, start_date):
        return {
            "marketplaceids": self.config.get('marketplace_ids'),
            "lastupdatedafter": start_date,
            "max_results": "50"
        }

    def parse_order_item(self, r):
        return {
            # Ids
            "id": pluck(r, ['AmazonOrderId', 'value']),
            "SellerOrderId": pluck(r, ['SellerOrderId', 'value']),
            "AmazonOrderId": pluck(r, ['AmazonOrderId', 'value']),
            "MarketplaceId": pluck(r, ['MarketplaceId', 'value']),

            # Strings
            "OrderStatus": pluck(r, ['OrderStatus', 'value']),
            "OrderType": pluck(r, ['OrderType', 'value']),
            "BuyerName": pluck(r, ['BuyerName', 'value']),
            "BuyerEmail": pluck(r, ['BuyerEmail', 'value']),
            "ShipServiceLevel": pluck(r, ['ShipServiceLevel', 'value']),
            "ShipServiceLevelCategory": pluck(r, ['ShipServiceLevelCategory', 'value']),
            "SalesChannel": pluck(r, ['SalesChannel', 'value']),
            "FulfillmentChannel": pluck(r, ['FulfillmentChannel', 'value']),
            "PaymentMethod": pluck(r, ['PaymentMethod', 'value']),

            # Dates
            "PurchaseDate": pluck(r, ['PurchaseDate', 'value']),
            "EarliestShipDate": pluck(r, ['EarliestShipDate', 'value']),
            "LatestShipDate": pluck(r, ['LatestShipDate', 'value']),
            "LastUpdateDate": pluck(r, ['LastUpdateDate', 'value']),

            # Counts
            "NumberOfItemsShipped": pluck(r, ['NumberOfItemsShipped', 'value']),
            "NumberOfItemsUnshipped": pluck(r, ['NumberOfItemsUnshipped', 'value']),

            # bools
            "IsReplacementOrder": pluck(r, ['IsReplacementOrder', 'value']),
            "IsBusinessOrder": pluck(r, ['IsBusinessOrder', 'value']),
            "IsPrime": pluck(r, ['IsPrime', 'value']),
            "IsPremiumOrder": pluck(r, ['IsPremiumOrder', 'value']),

            # Structs
            "ShippingAddress": {
                "City": pluck(r, ['ShippingAddress', 'City', 'value']),
                "PostalCode": pluck(r, ['ShippingAddress', 'PostalCode', 'value']),
                "StateOrRegion": pluck(r, ['ShippingAddress', 'StateOrRegion', 'value']),
                "CountryCode": pluck(r, ['ShippingAddress', 'CountryCode', 'value']),
                "Name": pluck(r, ['ShippingAddress', 'Name', 'value']),
                "AddressLine1": pluck(r, ['ShippingAddress', 'AddressLine1', 'value']),
                "AddressLine2": pluck(r, ['ShippingAddress', 'AddressLine2', 'value'])
            },

            # Prices
            "OrderTotal": get_price(r, 'OrderTotal'),
            "ShippingDiscount": get_price(r, 'ShippingDiscount'),
            "PromotionDiscount": get_price(r, 'PromotionDiscount')
        }

    def parse_order(self, r):
        return {
            # Ids
            "id": pluck(r, ['AmazonOrderId', 'value']),
            "SellerOrderId": pluck(r, ['SellerOrderId', 'value']),
            "AmazonOrderId": pluck(r, ['AmazonOrderId', 'value']),
            "MarketplaceId": pluck(r, ['MarketplaceId', 'value']),

            # Strings
            "OrderStatus": pluck(r, ['OrderStatus', 'value']),
            "OrderType": pluck(r, ['OrderType', 'value']),
            "BuyerName": pluck(r, ['BuyerName', 'value']),
            "BuyerEmail": pluck(r, ['BuyerEmail', 'value']),
            "ShipServiceLevel": pluck(r, ['ShipServiceLevel', 'value']),
            "ShipServiceLevelCategory": pluck(r, ['ShipServiceLevelCategory', 'value']),
            "SalesChannel": pluck(r, ['SalesChannel', 'value']),
            "FulfillmentChannel": pluck(r, ['FulfillmentChannel', 'value']),
            "PaymentMethod": pluck(r, ['PaymentMethod', 'value']),

            # Dates
            "PurchaseDate": pluck(r, ['PurchaseDate', 'value']),
            "EarliestShipDate": pluck(r, ['EarliestShipDate', 'value']),
            "LatestShipDate": pluck(r, ['LatestShipDate', 'value']),
            "LastUpdateDate": pluck(r, ['LastUpdateDate', 'value']),

            # Counts
            "NumberOfItemsShipped": pluck(r, ['NumberOfItemsShipped', 'value']),
            "NumberOfItemsUnshipped": pluck(r, ['NumberOfItemsUnshipped', 'value']),

            # bools
            "IsReplacementOrder": pluck(r, ['IsReplacementOrder', 'value']),
            "IsBusinessOrder": pluck(r, ['IsBusinessOrder', 'value']),
            "IsPrime": pluck(r, ['IsPrime', 'value']),
            "IsPremiumOrder": pluck(r, ['IsPremiumOrder', 'value']),

            # Structs
            "ShippingAddress": {
                "City": pluck(r, ['ShippingAddress', 'City', 'value']),
                "PostalCode": pluck(r, ['ShippingAddress', 'PostalCode', 'value']),
                "StateOrRegion": pluck(r, ['ShippingAddress', 'StateOrRegion', 'value']),
                "CountryCode": pluck(r, ['ShippingAddress', 'CountryCode', 'value']),
                "Name": pluck(r, ['ShippingAddress', 'Name', 'value']),
                "AddressLine1": pluck(r, ['ShippingAddress', 'AddressLine1', 'value']),
                "AddressLine2": pluck(r, ['ShippingAddress', 'AddressLine2', 'value'])
            },

            # Prices
            "OrderTotal": get_price(r, 'OrderTotal'),
            "ShippingDiscount": get_price(r, 'ShippingDiscount'),
            "PromotionDiscount": get_price(r, 'PromotionDiscount'),
        }

    def sync_order_items(self, order_id):
        order_items = self.client.fetch_order_items(order_id)
        time.sleep(1)

        parsed = []
        for o in order_items:
            parsed.append({
                "QuantityOrdered": pluck(o, ['QuantityOrdered', 'value']),
                "QuantityShipped": pluck(o, ['QuantityShipped', 'value']),
                "Title": pluck(o, ['Title', 'value']),
                "IsGift": pluck(o, ['IsGift', 'value']),
                "ASIN": pluck(o, ['ASIN', 'value']),
                "SellerSKU": pluck(o, ['SellerSKU', 'value']),
                "OrderItemId": pluck(o, ['OrderItemId', 'value']),
                "IsTransparency": pluck(o, ['IsTransparency', 'value']),
                "ProductInfo": {
                    "NumberOfItems": pluck(o, ['ProductInfo', 'NumberOfItems', 'value']),
                    "SerialNumberRequired": pluck(o, ['ProductInfo', 'SerialNumberRequired', 'value']),
                },
                "ItemPrice": get_price(o, 'ItemPrice'),
                "ItemTax": get_price(o, 'ItemTax'),
                "PromotionDiscount": get_price(o, 'PromotionDiscount'),
                "PromotionDiscountTax": get_price(o, 'PromotionDiscountTax'),
            })

        return parsed

    def get_stream_data(self, result):
        parsed = result.parsed
        for path in ['Orders', 'Order']:
            if path not in parsed:
                parsed = []
                break
            else:
                parsed = parsed.get(path)

        # Shove this into a list if its a dict
        # This can happen when only one record is returned by the API
        #   because the XML response does not encode list/dict info
        if isinstance(parsed, dict):
            parsed = [parsed]

        if len(parsed) > 0:
            LOGGER.info("Extracting data from {} orders ({} - {})".format(
                len(parsed), parsed[0]['LastUpdateDate'], parsed[-1]['LastUpdateDate']
            ))
        else:
            LOGGER.info("No orders returned!")

        records = []
        for record in parsed:
            parsed_record = self.parse_order(record)
            parsed_record["OrderItems"] = self.sync_order_items(parsed_record['AmazonOrderId'])
            records.append(self.transform_record(parsed_record))

        if len(records) > 0:
            LOGGER.info('DEBUG: last record is from: {}'.format(records[-1]['LastUpdateDate']))
        return records

    def sync_records(self, request_config, end_date=None):
        table = self.TABLE
        raw_orders =  self.client.fetch_orders(request_config)
        orders = self.get_stream_data(raw_orders)

        with singer.metrics.record_counter(endpoint=table) as counter:
            singer.write_records(table, orders)
            counter.increment(len(orders))

        if len(orders) > 0:
            state_key = 'LastUpdateDate'
            last_order = orders[-1]
            order_time = last_order[state_key]
            self.state = incorporate(self.state, self.TABLE, state_key, order_time)
            save_state(self.state)

            parsed = parse(order_time).date()
            if end_date is not None and parsed > end_date:
                LOGGER.info("Synced past the specified end_date ({}) - quitting".format(parsed))
                return None, orders

        next_token = raw_orders.parsed.get('NextToken', {}).get('value')
        return next_token, orders



from tap_amazon_mws.streams.base import InventoryIterationStream, pluck, get_price
from tap_amazon_mws.state import incorporate, save_state

from dateutil.parser import parse

import singer
import json
import time
import mws


LOGGER = singer.get_logger()  # noqa


class ProductStream(InventoryIterationStream):
    TABLE = 'products'
    KEY_PROPERTIES = ['id']

    def __init__(self, *args, **kwargs):
        super(ProductStream, self).__init__(*args, **kwargs)

    def get_config(self, product_id):
        return {
            "marketplaceid": self.config.get('marketplace_ids')[0],
            "type_": "SellerSKU",
            "ids": [product_id]
        }

    def parse_product(self, r):
        return {
            # Ids
            'id': pluck(r, ['Id', 'value']),
            'IdType': pluck(r, ['IdType', 'value']),
            'Product': pluck(r, ['Products', 'Product'])
        }

    def get_stream_data(self, result):
        parsed = result.parsed
        LOGGER.info("Parsing data from product")
        parsed_record = self.parse_product(parsed)

        try:
            return self.transform_record(parsed_record)
        except Exception as e:
            if hasattr(parsed, 'Id'):
                LOGGER.info("WARNING: Couldn't sync product with SellerSKU={}; {}".format(parsed.Id, e))
            else:
                LOGGER.info("WARNING: Couldn't sync product {}; {}".format(parsed, e))
            return None

    def sync_records(self, request_config, end_date=None):
        table = self.TABLE
        raw_product =  self.client.fetch_products(request_config)
        product = self.get_stream_data(raw_product)

        if product is not None:
            with singer.metrics.record_counter(endpoint=table) as counter:
                singer.write_records(table, [product])
                counter.increment()

        return product




from tap_amazon_mws.streams.orders import OrdersStream
from tap_amazon_mws.streams.inventory import InventoryStream
from tap_amazon_mws.streams.products import ProductStream

AVAILABLE_STREAMS = [
    OrdersStream,
    InventoryStream,
    ProductStream
]

__all__ = [
    'OrdersStream',
    'InventoryStream',
    'ProductStream',
]



from tap_amazon_mws.streams.base import PaginatedStream, pluck, get_price
from tap_amazon_mws.state import incorporate, save_state
from tap_amazon_mws.cache import InventoryCache

from dateutil.parser import parse

import singer
import json
import time
import mws


LOGGER = singer.get_logger()  # noqa

DEFAULT_INVENTORY_START_DATE = "2001-01-01"


class InventoryStream(PaginatedStream):
    TABLE = 'inventory'
    KEY_PROPERTIES = ['id']

    def __init__(self, *args, **kwargs):
        super(InventoryStream, self).__init__(*args, **kwargs)

    def get_config(self, _):
        start_date = self.config.get("inventory_start_date", DEFAULT_INVENTORY_START_DATE)
        start_date_dt = parse(start_date).date()

        return {
            "datetime_": start_date_dt,
            "response_group": "Basic"
        }

    def parse_inventory_item(self, r):
        return {
            # Ids
            "id": pluck(r, ['SellerSKU', 'value']),
            "ASIN": pluck(r, ['ASIN', 'value']),
            "FNSKU": pluck(r, ['FNSKU', 'value']),
            "SellerSKU": pluck(r, ['SellerSKU', 'value']),

            "Condition": pluck(r, ['Condition', 'value']),
            "SupplyDetail": pluck(r, ['SupplyDetail', 'value']),
            "TotalSupplyQuantity": pluck(r, ['TotalSupplyQuantity', 'value']),
            "InStockSupplyQuantity": pluck(r, ['InStockSupplyQuantity', 'value']),
            "EarliestAvailability": pluck(r, ['EarliestAvailability', 'TimepointType', 'value']),
        }

    def get_stream_data(self, result):
        parsed = result.parsed
        inventory_supply_list = parsed.get('InventorySupplyList', {})
        inventory_list = inventory_supply_list.get('member', [])

        # Shove this into a list if its a dict
        # This can happen when only one record is returned by the API
        #   because the XML response does not encode list/dict info
        if isinstance(inventory_list, dict):
            inventory_list = [inventory_list]

        LOGGER.info("Parsing data from {} inventory records".format(len(inventory_list)))

        records = []
        for record in inventory_list:
            parsed_record = self.parse_inventory_item(record)
            records.append(self.transform_record(parsed_record))

        return records

    def sync_records(self, request_config, end_date=None):
        table = self.TABLE
        raw_inventory =  self.client.fetch_inventory(request_config)
        inventory = self.get_stream_data(raw_inventory)

        with singer.metrics.record_counter(endpoint=table) as counter:
            singer.write_records(table, inventory)
            counter.increment(len(inventory))

        for record in inventory:
            InventoryCache[record['id']] = record

        next_token = raw_inventory.parsed.get('NextToken', {}).get('value')
        return next_token, inventory




import singer

from dateutil.parser import parse

LOGGER = singer.get_logger()  # noqa


def get_config_start_date(config):
    return parse(config.get('start_date')).date()


def get_config_end_date(config):
    end_date = config.get('end_date')
    if end_date:
        return parse(end_date).date()
    return None



#!/usr/bin/env python3

import singer

import tap_framework

from tap_amazon_mws.client import AmazonMWSClient
from tap_amazon_mws.streams import AVAILABLE_STREAMS

LOGGER = singer.get_logger()  # noqa


class AmazonMWSRunner(tap_framework.Runner):
    pass


@singer.utils.handle_top_exception(LOGGER)
def main():
    args = singer.utils.parse_args(
        required_config_keys=['access_key', 'secret_key', 'seller_id',
                              'region', 'marketplace_ids', 'start_date'])

    client = AmazonMWSClient(args.config)
    runner = AmazonMWSRunner(
        args, client, AVAILABLE_STREAMS)

    if args.discover:
        runner.do_discover()
    else:
        runner.do_sync()


if __name__ == '__main__':
    main()



import singer
import singer.utils
import singer.metrics

from tap_amazon_mws.cache import InventoryCache
from tap_amazon_mws.config import get_config_start_date, get_config_end_date
from tap_amazon_mws.state import get_last_record_value_for_table

from tap_framework.streams import BaseStream as base

LOGGER = singer.get_logger()


def pluck(r, path):
    for el in path:
        r = r.get(el)
        if r is None:
            return None
    return r


def get_price(record, key):
    if key in record:
        return {
            "CurrencyCode": pluck(record, [key, "CurrencyCode", 'value']),
            "Amount": pluck(record, [key, "Amount", 'value']),
        }
    else:
        return None


class BaseStream(base):
    KEY_PROPERTIES = ['id']

    def sync_data(self):
        pass

class PaginatedStream(BaseStream):

    def sync_data(self):
        table = self.TABLE
        page = 0

        start_date = get_last_record_value_for_table(self.state, table)
        if start_date is None:
            start_date = get_config_start_date(self.config)

        end_date = get_config_end_date(self.config)

        LOGGER.info('Syncing data for entity {} from {} (page={})'.format(table, start_date, page))

        next_token, records = self.sync_records(self.get_config(start_date), end_date=end_date)

        while next_token is not None:
            page += 1
            LOGGER.info('Syncing data for entity {} (page={})'.format(table, page))
            next_token, records = self.sync_records({"next_token": next_token}, end_date=end_date)

        return self.state


class InventoryIterationStream(BaseStream):

    def sync_data(self):
        table = self.TABLE

        LOGGER.info('Syncing data for {} {}'.format(len(InventoryCache), table))

        for i, product_id in enumerate(sorted(InventoryCache)):
            LOGGER.info('Syncing data for product {} of {} {}'.format(i + 1, len(InventoryCache), product_id))
            self.sync_records(self.get_config(product_id=product_id))

        return self.state



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(name='tap-amazon-mws',
      version='0.0.1',
      description='Singer.io tap for extracting data from the Amazon MWS API',
      author='Fishtown Analytics',
      url='http://fishtownanalytics.com',
      classifiers=['Programming Language :: Python :: 3 :: Only'],
      py_modules=['tap_amazon_mws'],
      install_requires=[
          'tap-framework==0.0.4',
          'mws',
      ],
      entry_points='''
          [console_scripts]
          tap-amazon-mws=tap_amazon_mws:main
      ''',
      packages=find_packages(),
      package_data={
          'tap_amazon_mws': [
              'schemas/*.json'
          ]
      })



import singer
import singer.metrics

import mws
import time


LOGGER = singer.get_logger()  # noqa


class AmazonMWSClient:

    MAX_TRIES = 5

    def __init__(self, config):
        self.config = config

        self.orders_api = self.get_orders_api()
        self.inventory_api = self.get_inventory_api()
        self.products_api = self.get_products_api()

    def get_orders_api(self):
        return mws.Orders(
            access_key=self.config.get('access_key'),
            secret_key=self.config.get('secret_key'),
            account_id=self.config.get('seller_id'),
            region=self.config.get('region'),
        )

    def get_inventory_api(self):
        return mws.Inventory(
            access_key=self.config.get('access_key'),
            secret_key=self.config.get('secret_key'),
            account_id=self.config.get('seller_id'),
            region=self.config.get('region'),
        )

    def get_products_api(self):
        return mws.Products(
            access_key=self.config.get('access_key'),
            secret_key=self.config.get('secret_key'),
            account_id=self.config.get('seller_id'),
            region=self.config.get('region'),
        )

    def obey_rate_limits(self, timeout=2):
        time.sleep(timeout)

    def fetch_orders(self, request_config):
        exc = None

        for i in range(self.MAX_TRIES):
            self.obey_rate_limits()
            try:
                return self.orders_api.list_orders(**request_config)
            except mws.mws.MWSError as e:
                exc = e
                LOGGER.info("Encountered an error while fetching orders, sleeping")
                LOGGER.error(e)
                time.sleep(60 * (i + 1))
        else:
            LOGGER.info("Failed after {} queries - raising".format(self.MAX_TRIES))
            raise exc

    def _fetch_order_items(self, request_config):
        exc = None

        for i in range(self.MAX_TRIES):
            self.obey_rate_limits(2)
            try:
                return self.orders_api.list_order_items(**request_config)
            except mws.mws.MWSError as e:
                exc = e
                LOGGER.info("Encountered an error while fetching order items, sleeping")
                LOGGER.error(e)
                time.sleep(20 * (i + 1))
        else:
            LOGGER.info("Failed after {} queries - raising".format(self.MAX_TRIES))
            raise exc

    def handle_order_items(self, resp):
        if isinstance(resp, dict):
            return [resp]
        else:
            return resp

    def fetch_order_items(self, order_id):
        page = 0
        LOGGER.info("Fetching order items for order={}, page={}".format(order_id, page))

        order_items = []
        resp = self._fetch_order_items({"amazon_order_id": order_id})
        while True:
            self.obey_rate_limits()
            new_items = resp.parsed.get('OrderItems', {}).get('OrderItem', [])
            order_items.extend(self.handle_order_items(new_items))

            next_token = resp.parsed.get('NextToken', {}).get('value')
            if next_token:
                page += 1
                LOGGER.info("Fetching order items for order={}, page={}".format(order_id, page))
                resp = self._fetch_order_items({"next_token": next_token})
            else:
                break

        return order_items

    def fetch_inventory(self, request_config):
        exc = None

        for i in range(self.MAX_TRIES):
            self.obey_rate_limits()
            try:
                return self.inventory_api.list_inventory_supply(**request_config)
            except mws.mws.MWSError as e:
                exc = e
                LOGGER.info("Encountered an error while fetching inventory, sleeping")
                LOGGER.error(e)
                time.sleep(60 * (i + 1))
        else:
            LOGGER.info("Failed after {} queries - raising".format(self.MAX_TRIES))
            raise exc

    def fetch_products(self, request_config):
        exc = None

        for i in range(self.MAX_TRIES):
            self.obey_rate_limits()
            try:
                return self.products_api.get_matching_product_for_id(**request_config)
            except mws.mws.MWSError as e:
                exc = e
                LOGGER.info("Encountered an error while fetching products, sleeping")
                LOGGER.error(e)
                time.sleep(60 * (i + 1))
        else:
            LOGGER.info("Failed after {} queries - raising".format(self.MAX_TRIES))
            raise exc
